<?php
/**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * Layout class
 *
 * Requirements:
 * @con_php_req 5.0
 *
 *
 * @package    Contenido Backend classes
 * @version    1.1
 * @author     Timo Hummel
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 *
 * {@internal
 *   created  2004-08-07
 *   modified 2011-03-15, Murat Purc, adapted to new GenericDB, partly ported to PHP 5, formatting
 *   modified 2018-04-05, René Mansveld, implemented editing in files
 *
 *   $Id$:
 * }}
 *
 */

if (!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}


class cApiLayoutCollection extends ItemCollection
{
    public function __construct()
    {
        global $cfg;
        parent::__construct($cfg["tab"]["lay"], "idlay");
        $this->_setItemClass("cApiLayout");
    }

    /** @deprecated  [2011-03-15] Old constructor function for downwards compatibility */
    public function cApiLayoutCollection()
    {
        cWarning(__FILE__, __LINE__, "Deprecated method call, use __construct()");
        $this->__construct();
    }

    public function create($title)
    {
        global $client;
        $item = parent::create();
        $item->set("name", $title);
        $item->set("idclient", $client);
        $item->store();
        return ($item);
    }
}


class cApiLayout extends Item
{
    protected $_error;

    protected $_bCodeFromFile = false;
    
    private $_aLayFileEditConf = array(        
                                        'use' => false,
                                        'folder_name' => 'layouts'
                                    );

    private $_bNoted = false;

    /**
     * Constructor Function
     * @param  mixed  $mId  Specifies the ID of item to load
     */
    public function __construct($mId = false)
    {
        global $cfg, $cfgClient, $client;
        parent::__construct($cfg["tab"]["lay"], "idlay");
        
        $this->setFilters(array(), array());
        
        if ((isset($cfg['dceModEdit'])) && (is_array($cfg['dceModEdit']))) {
            $this->_aLayFileEditConf = array_merge($this->_aLayFileEditConf, $cfg['dceModEdit']);
        }
        
        $oClient = new cApiClient($client);
        $aClientProp = $oClient->getPropertiesByType('modules_in_files');
        if (count($aClientProp)) {
            $this->_aLayFileEditConf = array_merge($this->_aLayFileEditConf, $aClientProp);
        }
        $this->_aLayFileEditConf['use'] = in_array($this->_aLayFileEditConf['use'], array('true', '1'));
        if (($this->_aLayFileEditConf['use']) && ((!isset($this->_aLayFileEditConf['full_path'])) || (empty($this->_aLayFileEditConf['full_path'])))) {
            $this->_aLayFileEditConf['full_path'] = $cfgClient[$client]["path"]["frontend"] . 'data/' . $this->_aLayFileEditConf['folder_name'];
            if (!is_dir($this->_aLayFileEditConf['full_path'])) {
                mkdir($this->_aLayFileEditConf['full_path'], 0777, true);
            }
        }
        $this->_aLayFileEditConf['full_path'] .= ((substr($this->_aLayFileEditConf['full_path'], -1) == '/') ? '' : '/');
        
        if ($mId !== false) {
            $this->loadByPrimaryKey($mId);
        }
    }

    /** @deprecated  [2011-03-15] Old constructor function for downwards compatibility */
    public function cApiLayout($mId = false)
    {
        cWarning(__FILE__, __LINE__, "Deprecated method call, use __construct()");
        $this->__construct($mId);
    }

    public function getField($field)
    {
        $value = parent::getField($field);

        switch ($field) {
            case "name":
                if ($value == "") {
                    $value = i18n("-- Unnamed Layout --");
                }
        }
        return ($value);
    }

    public function store($bJustStore = false)
    {
        global $cfg;
        
        if ($bJustStore) {
            // Just store changes, e.g. if specifying the mod package
            parent::store();
        } else {
            cInclude("includes", "functions.con.php");

            /* dceModFileEdit (c)2009-2011 www.dceonline.de */
            if ($this->_aLayFileEditConf['use']) {
                $this->modifiedValues['code'] = true;
            }
            /* End dceModFileEdit (c)2009-2011 www.dceonline.de */
            
            parent::store();

            conGenerateCodeForAllartsUsingLayout($this->get("idlay"));
        }
    }
    
    public function getLayFileEditConf() {
        return $this->_aLayFileEditConf;
    }

    /* dceModFileEdit (c)2009-2012 www.dceonline.de */
    
    /**
     * Overridden parent method for hooking dceModFileEdit
     * 
     * @return void
     */
    protected function _onLoad() {
        $this->_setCodeFromPhpFile();
    }

    /**
     * Use a PHP-file, if present, for layout code
     * 
     * @return boolean
     */
    private function _setCodeFromPhpFile () {
        // dceModEdit not enabled or not present
        if (!$this->_aLayFileEditConf['use']) {
            return false;
        }

        cInclude('includes', 'functions.upl.php');
        $sCodeFile = $this->_aLayFileEditConf['full_path'] . uplCreateFriendlyName($this->get('name')) . '.php';
        return $this->_setFieldFromFile($sCodeFile, 'code');
    }
    
    private function _displayNoteFromFile() {
        if ($this->_bNoted === true) {
            return;
        }
        if ($_GET['action'] == 'lay_sync_and_delete') {
            return;
        }
        
        global $frame, $area;
        if (($frame == 4) && ($area == 'lay_edit')) {
            $this->_bNoted = true;
            $oNote = new Contenido_Notification();
            $oNote->displayNotification('warning', i18n("Layout uses a file. Editing and Saving may not be possible in backend."));
        }
    }

    /**
     * read file and set an object field
     * 
     * @param string $sFile 
     * @param string $Field
     * @return boolean
     */
    private function _setFieldFromFile($sFile, $sField) {

        if ((is_file($sFile)) && (is_readable($sFile))) {
            $iFileSize = (int) filesize($sFile);
            if (($iFileSize) && ($fh = fopen($sFile, 'r'))) {
                $this->set($sField, fread($fh, $iFileSize), false);
                fclose($fh);
            } else {
                $this->set($sField, '');
            }
            switch($sField) {
                case "code":
                    $this->_bCodeFromFile = true;
                    $this->_displayNoteFromFile();
                    break;
            }
            return true;
        }        
        return false;
    }
    
    public function isLoadedFromFile($sWhat) {
        switch($sWhat) {
            case "code":
                return $this->_bCodeFromFile;
                break;
            
            default:
                return false;
        }
    }
    /* End dceModFileEdit (c)2009-2012 www.dceonline.de */
    
    public function getFullPath() {
        global $cfgClient, $client;
        
        if (!$this->_aLayFileEditConf['use']) {
            return false;
        }
        return str_replace(dirname($cfgClient[$client]["path"]["frontend"]), '', $this->_aLayFileEditConf['full_path']) . uplCreateFriendlyName($this->get('name') . '.php');
    }
}

?>