<?php
/**
 * Includes globals array access class.
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @date        07.10.2008
 * @package     Contenido
 * @subpackage  Globals
 */


defined('CON_FRAMEWORK') or die('Illegal call');


include_once('class.mparraymanager.php');


/**
 * Globals array ($GLOBALS) manager class. Provides another way to access (reading/writing) to
 * globals array.
 *
 * Number of access deepness to the array structure depends on extended class 'mpArrayManager'.
 * This is by default up to 4 levels, e. g. $arr['one']['two']['three']['four'].
 *
 * The whole purpose of this class is to replace usage of superglobal $GLOBALS and the global
 * declaration of variables (global $foobar;) within functions.
 * @see http://www.php.net/manual/en/language.variables.scope.php
 *
 *
 * Usage:
 * <code>
 * // example globals content
 * $GLOBALS['key']['subkey']['anotherkey'] = 123;
 *
 * // example access (assuming that access delemiter is /)
 * $glob = mpGlobals::getInstance();
 * echo $glob->get('key/subkey/anotherkey');
 *
 * // example for setting the content
 * $glob->set('key/subkey/anotherkey', 321);
 * </code>
 *
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  Globals
 */
class mpGlobals extends mpArrayManager implements mpIArrayManager {

    /**
     * Constructor, stores the reference to $GLOBALS into member variable _aData.
     */
    private function __construct() {
        $this->_aData = & $GLOBALS;
    }

    /**
     * Returns a instance of mpGlobals (singleton implementation)
     *
     * @return  mpGlobals
     */
    public static function getInstance() {
        if (self::$_instance == null) {
            self::$_instance = new mpGlobals();
        }
        return self::$_instance;
    }

}
