<?php
/**
 * Project: 
 * DZIRcms Content Management System
 * 
 * Description: 
 * Cron Job script for the Newsletter plugin.
 * Automatically send newsletter jobs in the background, send a status email to the
 * creator of the job when the job is finished.
 *
 * @package     Newslette Plugin
 * @author      René Mansveld
 * @copyright   Spider IT Deutschland
 * @license     https://www.dzir.org/cms/license/LIZENZ.txt
 * @link        https://www.dzir.org/cms/
 * @link        https://www.spider-it.de
 * @created     2021-01-11
 *
 * modified:
 * 
 */

if (!defined("CON_FRAMEWORK")) {
    define("CON_FRAMEWORK", true);
}

# DZIRcms startup process
include_once(str_replace('\\', '/', realpath(dirname(__FILE__) . '/../')) . '/includes/startup.php');

# Newsletter plugin specific startup process
plugin_include('newsletter', 'includes/config.plugin.php');

global $cfg;

# Check if the newsletter plugin was started correctly
if (!class_exists('cNewsletterJobCollection')) {
    exit();
}

#$area = 'cronjobs';

# Check for waiting jobs
$oJobs = new cNewsletterJobCollection();
$oJobs->setWhere("status", 1);
$oJobs->setWhere("use_cronjob", 1);
$oJobs->setLimit("0", "1");             # Load only one job at a time
$oJobs->setOrder("created ASC");        # Oldest job will be run first
$oJobs->query();

$bJobRun = false;
if ($oJob = $oJobs->next()) {
    
    # Active jobs found, run job
    $oJob->runJob();
    $bJobRun = true;
}
else {
    
    # Nothing to do, check dead jobs
    $oJobs->resetQuery();
    $oJobs->setWhere("status", 2);
    $oJobs->setWhere("use_cronjob", 1);
    $oJobs->setLimit("0", "1");         # Load only one job at a time
    $oJobs->setOrder("created ASC");    # Oldest job will be run first
    $oJobs->query();
    
    if ($oJob = $oJobs->next()) {
        
        # Maybe hanging jobs found, run job
        $oJob->runJob();
        $bJobRun = true;
    }
}
if ($bJobRun) {
    if ($oJob->get('status') == 9) {
        
        # Send an email to the creator of the job to inform him/her it's finished
        $sAuthor = $oJob->get('author');
        $sql = 'SELECT realname, email
                FROM ' . $cfg['tab']['user'] . '
                WHERE (user_id="' . $sAuthor . '")';
        $db = new DB();
        $db->query($sql);
        if ($db->next_record()) {
            
            # Use the job creator's name and email address
            $sRealName = $db->f('realname');
            $sEmail = $db->f('email');
        }
        else {
            
            # The job creator's account doesn't exist anymore, use it's saved name
            $sRealName = $oJob->get('authorname');
            $sEmail = '';
        }
        if (strlen($sEmail) == 0) {
            
            # No email address specified, use the sender email address from the job
            $sEmail = $oJob->get('newsfrom');
        }
        if (strlen($sEmail)) {
            
            # Get the number of successful newsletter sendings
            $sql = 'SELECT COUNT(*) AS successful
                    FROM ' . $cfg['tab']['news_log'] . '
                    WHERE ((idnewsjob=' . $oJob->get('idnewsjob') . ')
                       AND (status="successful"))';
            $db->query($sql);
            $db->next_record();
            $iSuccessful = $db->f('successful');
            
            # Get the number of failed newsletter sendings
            $sql = 'SELECT COUNT(*) AS error
                    FROM ' . $cfg['tab']['news_log'] . '
                    WHERE ((idnewsjob=' . $oJob->get('idnewsjob') . ')
                       AND (status LIKE "error%"))';
            $db->query($sql);
            $db->next_record();
            $iError = $db->f('error');
            
            # Instanciate the mailer
            $oMail = new PHPMailer();
            
            # Mailer Configuration
            $sMailer                = strtolower(getEffectiveSetting('newsletter', 'mailer'));
            $sHost                  = getEffectiveSetting('newsletter', 'host');
            $iPort                  = intval(getEffectiveSetting('newsletter', 'port'));
            $sUsername              = getEffectiveSetting('newsletter', 'username');
            $sPassword              = getEffectiveSetting('newsletter', 'password');
            if (strlen($sMailer) == 0) {
                $sMailer            = strtolower(getEffectiveSetting('email', 'mailer'));
                $sHost              = getEffectiveSetting('email', 'smtp-server');
                $iPort              = intval(getEffectiveSetting('email', 'smtp-port'));
                $sSslTls            = getEffectiveSetting('email', 'ssl/tls');
                $sUsername          = getEffectiveSetting('email', 'smtp-login');
                $sPassword          = getEffectiveSetting('email', 'smtp-password');
            }
            if (strlen($sMailer) == 0) {
                setClientProperty('email', 'mailer', 'mail');
                $sMailer = 'mail';
            }
            if ($sMailer == 'smtp') {
                if (strlen($sHost) == 0) {
                    setClientProperty('email', 'smtp-server', '');
                }
                if ($iPort == 0) {
                    setClientProperty('email', 'smtp-port', '25');
                    $iPort = 25;
                }
                if (strlen($sSslTls) == 0) {
                    setClientProperty('email', 'ssl/tls', '');
                }
                if (strlen($sUsername) == 0) {
                    setClientProperty('email', 'smtp-login', '');
                }
                if (strlen($sPassword) == 0) {
                    setClientProperty('email', 'smtp-password', '');
                }
                if (substr($sPassword, 0, 3) == '&#x') {
                    $sPassword = html_entity_decode($sPass, ENT_QUOTES, $encoding[$ilang]);
                }
            }
            $oLang = new cApiLanguage();
            $oLang->loadByPrimaryKey($lang);
            $oMail->setLanguage($oLang->getProperty('language', 'code'), $cfg['path']['contenido'] . $cfg['path']['external'] . 'PHPMailer/language/');
            $oMail->CharSet = $encoding[$lang];
            $oMail->IsHTML(false);
            $oMail->Mailer = $sMailer;
            if ($sMailer == 'smtp') {
                $oMail->SMTPAuth    = true;
                $oMail->Host        = $sHost;
                $oMail->Port        = $iPort;
                $oMail->SMTPSecure  = $sSslTls;
                $oMail->Username    = $sUsername;
                $oMail->Password    = $sPassword;
            }
            
            # Set the header and content of the mail
            $sFrom = $oJob->get('newsfrom');
            $oMail->From = $sFrom;
            $sFromName = $oJob->get('newsfromname');
            $oMail->FromName = $sFromName;
            $oMail->AddAddress($sEmail, $sRealName);
            $sDateTimeFormat = getEffectiveSetting('dateformat', 'full', 'Y-m-d H:i');
            $sSubject = 'Newsletter "' . $oJob->get('name') . ' (' . date($sDateTimeFormat, strtotime($oJob->get('newsdate'))) . ')' . '" was sent';
            $oMail->Subject = $sSubject;
            $sBody = 'Newsletter "' . $oJob->get('name') . ' (' . date($sDateTimeFormat, strtotime($oJob->get('newsdate'))) . ')' . '" was sent.' . "\n" . 'Successful: ' . $iSuccessful . ', failed: ' . $iError . '.' . "\n" . 'Job started: ' . date($sDateTimeFormat, strtotime($oJob->get('started'))) . ', finished: ' . date($sDateTimeFormat, strtotime($oJob->get('finished'))) . '.' . "\n" . 'Please check the job\'s details for more information.' . "\n\n";
            $oMail->Body = $sBody;
            $oMail->WordWrap = 76;
            
            # Send the mail
            $result = $oMail->Send();
            
            # Output the mail data to the browser if it was tested there
            if (isRunningFromWeb()) {
                echo '<pre>MailTo: ' . $sRealName . ' &lt;' . $sEmail . '&gt;</pre>';
                echo '<pre>MailFrom: ' . $sFromName . ' &lt;' . $sFrom . '&gt;</pre>';
                echo '<pre>Subject: ' . $sSubject . '</pre>';
                echo '<pre>Body:' . "\n" . $sBody . '</pre>';
                echo '<pre>Sent: ' . (($result) ? 'true' : 'false') . '</pre>';
            }
        }
    }
}