<?php
/**
 * Project: 
 * DZIRcms Content Management System
 * 
 * Description: 
 * Allows editing the created HTML before it get's compressed
 *
 * @package    DZIRcms Frontend
 * @version    1.0.0
 * @author     René Mansveld
 * @copyright  Spider IT Deutschland <www.spider-it.de>
 * @license    https://www.dzir.org/cms/license/
 * @link       https://www.dzir.org/cms
 * @link       http://www.spider-it.de
 * @since      file available since DZIRcms release 2.1.0
 */

if (!defined('CON_FRAMEWORK')) {
	die('Illegal call');
}

/**
 * Edit the $htmlCode variable, which holds the ready generated
 * page output. DO NOT echo the content, as it would be outputted
 * uncompressed with the compressed and optimized version following
 * it, disturbing the browser view.
 */

# This example replaces [TITLE] in html code with the content of the title tag
$p1 = strpos($htmlCode, '<title>');
if ($p1 !== false) {
    $p1 += strlen('<title>');
    $p2 = strpos($htmlCode, '</title>', $p1);
    $title = substr($htmlCode, $p1, ($p2 - $p1));
    $htmlCode = str_replace('[TITLE]', $title, $htmlCode);
}

# This example replaces [CANONICAL] in html code with the url of the canonical tag
$p1 = strpos($htmlCode, '<link rel="canonical"');
if ($p1 !== false) {
    $p1 = strpos($htmlCode, 'href="', $p1);
    $p1 += strlen('href="');
    $p2 = strpos($htmlCode, '"', $p1);
    $canonical = substr($htmlCode, $p1, ($p2 - $p1));
    $htmlCode = str_replace('[CANONICAL]', $canonical, $htmlCode);
}

# This example transforms email links and addresses to ascii code
function encodeMail($string) {
	$length = strlen($string) - 1;
    $result = '';
	for ($i = 0; $i <= $length; $i ++) {
		$char = ord($string[$i]);
		$result = $result . '&#' . $char . ';';
	}
    return $result;
}
# - Replace the email addresses
if (preg_match_all('!"(mailto:)([^"]+)"!i', $htmlCode, $addresses)) {
    foreach ($addresses[2] as $address) {
        $htmlCode = str_replace($address, encodeMail($address), $htmlCode);
    }
}
# - Replace the 'mailto:' text in the link
$htmlCode = str_replace('mailto:', encodeMail('mailto:'), $htmlCode);
?>