<?php
/**
 * Project: 
 * Contenido Content Management System
 * 
 * Description: 
 * Defines the Layout related functions
 * 
 * Requirements: 
 * @con_php_req 5.0
 * 
 *
 * @package    Contenido Backend includes
 * @version    1.3.2
 * @author     Jan Lengowski
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 * @since      file available since contenido release <= 4.6
 * 
 * {@internal 
 *   created 2003
 *   modified 2008-06-26, Frederic Schneider, add security fix
 *   modified 2018-04-05, René Mansveld, implemented editing in files
 *
 *   $Id$:
 * }}
 * 
 */

if(!defined('CON_FRAMEWORK')) {
	die('Illegal call');
}

cInclude ("includes", "functions.tpl.php");
cInclude ("includes", "functions.con.php");

/**
 * Edit or Create a new layout
 *
 * @param int $idlay Id of the Layout
 * @param string $name Name of the Layout
 * @param string $description Description of the Layout
 * @param string $code Layout HTML Code
 * @return int $idlay Id of the new or edited Layout
 *
 * @author Olaf Niemann <olaf.niemann@4fb.de>
 * @copryright four for business AG <www.4fb.de>
 */
function layEditLayout($idlay, $name, $description, $code) {

    global $db, $client, $cfgClient, $auth, $cfg, $sess, $area_tree, $perm;

    $date = date("Y-m-d H:i:s");
    $author = $auth->auth["uname"];
    $description = (string) stripslashes($description);
    
    set_magic_quotes_gpc($name);
    set_magic_quotes_gpc($description);
    set_magic_quotes_gpc($code);
	
	/**
	* START TRACK VERSION
	**/
	$oVersion = new VersionLayout($idlay, $cfg, $cfgClient, $db, $client, $area, $frame);
	
	// Create new Layout Version in cms/version/layout/
	$oVersion->createNewVersion();
    
	/**
	* END TRACK VERSION
	**/
	
    if (!$idlay) {
        $cApiLayoutCollection = new cApiLayoutCollection();
        $cApiLayout = $cApiLayoutCollection->create($name);
        
        $idlay = $cApiLayout->get('idlay');
        
        // set correct rights for element
        cInclude ("includes", "functions.rights.php");
        createRightsForElement("lay", $idlay);
    } else {
        $cApiLayout = new cApiLayout($idlay);
    }
    
    /* dceModFileEdit (c)2009-2011 www.dceonline.de */
    if ((in_array(getEffectiveSetting('modules_in_files', 'use', 'false'), array('true', '1')))
            || ($cApiLayout->get("name") != stripslashes($name))
            || ($cApiLayout->get("code") != stripslashes($code))
            || ($cApiLayout->get("description") != stripslashes($description))) {
        
        $cApiLayout->set("name", $name);
        $cApiLayout->set("code", $code);
        $cApiLayout->set("description", $description);
        $cApiLayout->set("lastmodified", $date);
        
        $cApiLayout->store();
    }	
    return $idlay;
}

function layDeleteLayout($idlay) {
    global $db, $client, $cfg, $area_tree, $perm;

    $sql = "SELECT * FROM ".$cfg["tab"]["tpl"]." WHERE idlay='".Contenido_Security::toInteger($idlay)."'";
    $db->query($sql);
    if ($db->next_record()) {
        return "0301"; // layout is still in use, you cannot delete it
    } else {
        $sql = "DELETE FROM ".$cfg["tab"]["lay"]." WHERE idlay='".Contenido_Security::toInteger($idlay)."'";
        $db->query($sql);

        // delete rights for element
        cInclude ("includes", "functions.rights.php");
        deleteRightsForElement("lay", $idlay); 
    }
}
?>