<?php
/**
 * Project: 
 * DZIRcms Content Management System
 * 
 * Description: 
 * Functions for the frontend navigation
 * 
 * @package     DZIRcms Backend includes
 * @version     1.0.0
 * @author      René Mansveld
 * @copyright   Spider IT Deutschland
 * @license     https://www.dzir.org/cms/license/
 * @link        https://www.dzir.org/cms/
 * @link        https://www.spider-it.de/
 * @since       file available since DZIRcms release 2.1.0
 * 
 */

/**
 * Functions in this file
 * 
 * FrontendNavigationListDropDown
 * A navigation with <ul> and <li> nested and non-selectable parent categories
 * 
 * function FrontendNavigationListDropDownButton
 * A navigation with <ul> and <li> nested and selectable parent categories, separate dropdown buttons
 * 
 * function FrontendNavigationDivDropDown
 * A navigation with <div> nested and non-selectable parent categories
 * 
 * function FrontendNavigationDivDropDownButton
 * A navigation with <div> nested and selectable parent categories, separate dropdown buttons
 */

/**
 * function FrontendNavigationListDropDown
 * 
 * A navigation with <ul> and <li> nested and non-selectable parent categories
 * 
 * @param integer $iCatRoot - $idcat of the parent category, defaults to 1
 * @param integer $iLevels - Number of levels to output, defaults to 2
 * @param string $sAdditionalClasses - Additional classes to add to the <li> elements ('{name}' adds the name)
 * @param boolean $bIncludeRootUl - Include the root <ul>, defaults to true
 * @param integer $_iLevel - (internal) The current level (counter for the number of levels to output)
 * @return string Navigation HTML code
 */
function FrontendNavigationListDropDown(int $iCatRoot = 1, int $iLevels = 2, string $sAdditionalClasses = '', bool $bIncludeRootUl = true, int $_iLevel = 1) {
    global $idcat, $client, $lang, $cfg;
    
    $db = new DB();
    $sResult = '';
    
    # Needed to mark active elements
    $sCurPath = '';
    $separator = '~|~';
    conCreateLocationString($idcat, $separator, $sCurPath);
    $aCurPath = explode($separator, $sCurPath);
        
    $sql = 'SELECT *, cl.public, al.redirect, al.external_redirect
            FROM ' . $cfg['tab']['cat_tree'] . ' ct INNER JOIN ' . $cfg['tab']['cat'] . ' c ON ct.idcat = c.idcat INNER JOIN ' . $cfg['tab']['cat_lang'] . ' cl ON c.idcat = cl.idcat INNER JOIN ' . $cfg['tab']['art_lang'] . ' al ON cl.startidartlang = al.idartlang
            WHERE ((cl.idlang=' . $lang . ')
               AND (c.idclient=' . $client . ')
               AND (c.parentid=' . $iCatRoot . ')
               AND (cl.visible=1))
            ORDER BY ct.idtree';
    $db->query($sql);
    $bHits = boolval($db->num_rows());
    if ($bHits) {
        if ((($_iLevel == 1) && ($bIncludeRootUl)) || ($_iLevel > 1)) {
            $sResult .= "\n" . '<ul class="navbar-nav level-X">';
        }
    }
    while ($db->next_record()) {
        $sResult = str_replace('level-X', 'level-' . $_iLevel, $sResult);
        $sResult .= "\n" . '<li class="navi_' . ((in_array($db->f('name'), $aCurPath)) ? 'active' : 'normal') . ((strlen($sAdditionalClasses)) ? ' ' . str_replace('{name}', str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))), $sAdditionalClasses) : '') . '">';
        $sSub = '';
        if ($_iLevel < $iLevels) {
            $sSub = FrontendNavigationListDropDown($db->f('idcat'), $iLevels, ($_iLevel + 1));
        }
        if (strlen($sSub)) {
            $sResult .= '<div class="button" data-name="' . str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))) . '">' . $db->f('name') . '</div><div id="' . str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))) . '" class="subnavi">' . $sSub . '</div>';
        }
        else {
            $sResult .= '<a href="front_content.php?idcat=' . $db->f('idcat') . '&lang=' . $lang . '">' . $db->f('name') . '</a>';
        }
        $sResult .= '</li>';
    }    
    if ($bHits) {
        if ((($_iLevel == 1) && ($bIncludeRootUl)) || ($_iLevel > 1)) {
            $sResult .= "\n" . '</ul>';
        }
    }
    
    $db->disconnect();
    
    return $sResult;
}

/**
 * function FrontendNavigationListDropDownButton
 * 
 * A navigation with <ul> and <li> nested and selectable parent categories, separate dropdown buttons
 * 
 * @param integer $iCatRoot - $idcat of the parent category, defaults to 1
 * @param integer $iLevels - Number of levels to output, defaults to 2
 * @param string $sAdditionalClasses - Additional classes to add to the <li> elements ('{name}' adds the name)
 * @param boolean $bIncludeRootUl - Include the root <ul>, defaults to true
 * @param integer $_iLevel - (internal) The current level (counter for the number of levels to output)
 * @return string Navigation HTML code
 */
function FrontendNavigationListDropDownButton(int $iCatRoot = 1, int $iLevels = 2, string $sAdditionalClasses = '', bool $bIncludeRootUl = true, int $_iLevel = 1) {
    global $idcat, $client, $lang, $cfg;
    
    $db = new DB();
    $sResult = '';
    
    # Needed to mark active elements
    $sCurPath = '';
    $separator = '~|~';
    conCreateLocationString($idcat, $separator, $sCurPath);
    $aCurPath = explode($separator, $sCurPath);
        
    $sql = 'SELECT *, cl.public, al.redirect, al.external_redirect
            FROM ' . $cfg['tab']['cat_tree'] . ' ct INNER JOIN ' . $cfg['tab']['cat'] . ' c ON ct.idcat = c.idcat INNER JOIN ' . $cfg['tab']['cat_lang'] . ' cl ON c.idcat = cl.idcat INNER JOIN ' . $cfg['tab']['art_lang'] . ' al ON cl.startidartlang = al.idartlang
            WHERE ((cl.idlang=' . $lang . ')
               AND (c.idclient=' . $client . ')
               AND (c.parentid=' . $iCatRoot . ')
               AND (cl.visible=1))
            ORDER BY ct.idtree';
    $db->query($sql);
    $bHits = boolval($db->num_rows());
    if ($bHits) {
        if ((($_iLevel == 1) && ($bIncludeRootUl)) || ($_iLevel > 1)) {
            $sResult .= "\n" . '<ul class="navbar-nav level-X">';
        }
    }
    while ($db->next_record()) {
        $sResult = str_replace('level-X', 'level-' . $_iLevel, $sResult);
        $sResult .= "\n" . '<li class="navi_' . ((in_array($db->f('name'), $aCurPath)) ? 'active' : 'normal') . ((strlen($sAdditionalClasses)) ? ' ' . str_replace('{name}', str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))), $sAdditionalClasses) : '') . '">';
        $sSub = '';
        if ($_iLevel < $iLevels) {
            $sSub = FrontendNavigationListDropDownButton($db->f('idcat'), $iLevels, ($_iLevel + 1));
        }
        if (strlen($sSub)) {
            $sResult .= '<span><a href="front_content.php?idcat=' . $db->f('idcat') . '&lang=' . $lang . '">' . $db->f('name') . '</a><span class="button" data-name="' . str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))) . '"></span></span><div id="' . str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))) . '" class="subnavi">' . $sSub . '</div>';
        }
        else {
            $sResult .= '<a href="front_content.php?idcat=' . $db->f('idcat') . '&lang=' . $lang . '">' . $db->f('name') . '</a>';
        }
        $sResult .= '</li>';
    }    
    if ($bHits) {
        if ((($_iLevel == 1) && ($bIncludeRootUl)) || ($_iLevel > 1)) {
            $sResult .= "\n" . '</ul>';
        }
    }
    
    $db->disconnect();
    
    return $sResult;
}

/**
 * function FrontendNavigationDivDropDown
 * 
 * A navigation with <div> nested and non-selectable parent categories
 * 
 * @param integer $iCatRoot - $idcat of the parent category, defaults to 1
 * @param integer $iLevels - Number of levels to output, defaults to 2
 * @param string $sAdditionalClasses - Additional classes to add to the <div> elements ('{name}' adds the name)
 * @param integer $_iLevel - (internal) The current level (counter for the number of levels to output)
 * @return string Navigation HTML code
 */
function FrontendNavigationDivDropDown(int $iCatRoot = 1, int $iLevels = 2, string $sAdditionalClasses = '', int $_iLevel = 1) {
    global $idcat, $client, $lang, $cfg;
    
    $db = new DB();
    $sResult = '';
    
    # Needed to mark active elements
    $sCurPath = '';
    $separator = '~|~';
    conCreateLocationString($idcat, $separator, $sCurPath);
    $aCurPath = explode($separator, $sCurPath);
        
    $sql = 'SELECT *, cl.public, al.redirect, al.external_redirect
            FROM ' . $cfg['tab']['cat_tree'] . ' ct INNER JOIN ' . $cfg['tab']['cat'] . ' c ON ct.idcat = c.idcat INNER JOIN ' . $cfg['tab']['cat_lang'] . ' cl ON c.idcat = cl.idcat INNER JOIN ' . $cfg['tab']['art_lang'] . ' al ON cl.startidartlang = al.idartlang
            WHERE ((cl.idlang=' . $lang . ')
               AND (c.idclient=' . $client . ')
               AND (c.parentid=' . $iCatRoot . ')
               AND (cl.visible=1))
            ORDER BY ct.idtree';
    $db->query($sql);
    while ($db->next_record()) {
        $sResult = str_replace('level-X', 'level-' . $_iLevel, $sResult);
        $sResult .= "\n" . '<div class="navi_' . ((in_array($db->f('name'), $aCurPath)) ? 'active' : 'normal') . ((strlen($sAdditionalClasses)) ? ' ' . str_replace('{name}', str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))), $sAdditionalClasses) : '') . '">';
        $sSub = '';
        if ($_iLevel < $iLevels) {
            $sSub = FrontendNavigationDivDropDown($db->f('idcat'), $iLevels, ($_iLevel + 1));
        }
        if (strlen($sSub)) {
            $sResult .= '<div class="button" data-name="' . str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))) . '">' . $db->f('name') . '</div><div id="' . str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))) . '" class="subnavi">' . $sSub . '</div>';
        }
        else {
            $sResult .= '<a href="front_content.php?idcat=' . $db->f('idcat') . '&lang=' . $lang . '">' . $db->f('name') . '</a>';
        }
        $sResult .= '</div>';
    }    
    
    $db->disconnect();
    
    return $sResult;
}

/**
 * function FrontendNavigationDivDropDownButton
 * 
 * A navigation with <div> nested and selectable parent categories, separate dropdown buttons
 * 
 * @param integer $iCatRoot - $idcat of the parent category, defaults to 1
 * @param integer $iLevels - Number of levels to output, defaults to 2
 * @param string $sAdditionalClasses - Additional classes to add to the <div> elements ('{name}' adds the name)
 * @param integer $_iLevel - (internal) The current level (counter for the number of levels to output)
 * @return string Navigation HTML code
 */
function FrontendNavigationDivDropDownButton(int $iCatRoot = 1, int $iLevels = 2, string $sAdditionalClasses = '', int $_iLevel = 1) {
    global $idcat, $client, $lang, $cfg;
    
    $db = new DB();
    $sResult = '';
    
    # Needed to mark active elements
    $sCurPath = '';
    $separator = '~|~';
    conCreateLocationString($idcat, $separator, $sCurPath);
    $aCurPath = explode($separator, $sCurPath);
        
    $sql = 'SELECT *, cl.public, al.redirect, al.external_redirect
            FROM ' . $cfg['tab']['cat_tree'] . ' ct INNER JOIN ' . $cfg['tab']['cat'] . ' c ON ct.idcat = c.idcat INNER JOIN ' . $cfg['tab']['cat_lang'] . ' cl ON c.idcat = cl.idcat INNER JOIN ' . $cfg['tab']['art_lang'] . ' al ON cl.startidartlang = al.idartlang
            WHERE ((cl.idlang=' . $lang . ')
               AND (c.idclient=' . $client . ')
               AND (c.parentid=' . $iCatRoot . ')
               AND (cl.visible=1))
            ORDER BY ct.idtree';
    $db->query($sql);
    while ($db->next_record()) {
        $sResult = str_replace('level-X', 'level-' . $_iLevel, $sResult);
        $sResult .= "\n" . '<div class="navi_' . ((in_array($db->f('name'), $aCurPath)) ? 'active' : 'normal') . ((strlen($sAdditionalClasses)) ? ' ' . str_replace('{name}', str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))), $sAdditionalClasses) : '') . '">';
        $sSub = '';
        if ($_iLevel < $iLevels) {
            $sSub = FrontendNavigationDivDropDownButton($db->f('idcat'), $iLevels, ($_iLevel + 1));
        }
        if (strlen($sSub)) {
            $sResult .= '<span><a href="front_content.php?idcat=' . $db->f('idcat') . '&lang=' . $lang . '">' . $db->f('name') . '</a><span class="button" data-name="' . str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))) . '">' . $db->f('name') . '</span></span><div id="' . str_replace(array(' ', '-', '_'), '', capiStrReplaceDiacritics($db->f('name'))) . '" class="subnavi">' . $sSub . '</div>';
        }
        else {
            $sResult .= '<a href="front_content.php?idcat=' . $db->f('idcat') . '&lang=' . $lang . '">' . $db->f('name') . '</a>';
        }
        $sResult .= '</div>';
    }    
    
    $db->disconnect();
    
    return $sResult;
}
?>