<?php
 /**
 * Project:
 * DZIRcms Content Management System
 *
 * Description:
 * DZIRcms setup script
 *
 * Requirements:
 * @con_php_req 7
 *
 * @package    DZIRcms setup
 * @version    2.0.1
 * @author     unknown
 * @copyright  Spider IT Deutschland
 * @license    https://www.dzir.org/cms/license/
 * @link       https://www.dzir.ord/cms/
 * @link       http://www.spider-it.de
 * 
 *
 * {@internal
 *   created  unknown
 *   modified 2008-07-07, bilal arslan, added security fix
 *   modified 2011-01-21, Ortwin Pinke, added php-errorhandling function calls, uncomment if needed
 *   modified 2011-02-24, Murat Purc, extended mysql extension detection and some other changes
 *   modified 2011-02-28, Murat Purc, normalized setup startup process and some cleanup/formatting
 *   modified 2021-07-03, René Mansveld, reworked completely for DZIRcms
 *
 *   $Id$:
 * }}
 *
 */

if (!defined('CON_FRAMEWORK')) {
    define('CON_FRAMEWORK', true);
}
define('C_CONTENIDO_PATH', '../admin/');

include_once('lib/startup.php');


if (is_array($_REQUEST)) {
    foreach ($_REQUEST as $key => $value) {
        if (($value != '' && $key != 'dbpass') || ($key == 'dbpass' && $_REQUEST['dbpass_changed'] == 'true')) {
            $_SESSION[$key] = $value;
        }
    }
}


if (getPHPIniSetting('session.use_cookies') == 0) {
    $sNotInstallableReason = 'session_use_cookies';
    checkAndInclude('steps/notinstallable.php');
}

if (strlen($_SESSION['dbmode'])) {
    $cfg['database_extension'] = strtolower($_SESSION['dbmode']);
} elseif (hasMySQLExtension() && !hasMySQLiExtension()) {
    // use MySQL extension by default if available
    $cfg['database_extension'] = 'mysql';
} elseif (hasMySQLiExtension()) {
    // use MySQLi extension if available
    $cfg['database_extension'] = 'mysqli';
} else {
    $sNotInstallableReason = 'database_extension';
    checkAndInclude('steps/notinstallable.php');
}

checkAndInclude('../conlib/prepend.php');

i18nRegisterDomain("setup", 'locale/');
if (array_key_exists('language', $_SESSION)) {
    i18nInit('locale/', $_SESSION['language']);
}
if (phpversion() < C_SETUP_MIN_PHP_VERSION) {
    $sNotInstallableReason = 'php_version';
    checkAndInclude('steps/notinstallable.php');
}

if (array_key_exists('step', $_REQUEST)) {
    $iStep = $_REQUEST['step'];
} else {
    $iStep = '';
}

switch ($iStep) {
    case 'setuptype':
        checkAndInclude('steps/setuptype.php');
        break;
    case 'setup1':
        checkAndInclude('steps/setup/step1.php');
        break;
    case 'setup2':
        checkAndInclude('steps/setup/step2.php');
        break;
    case 'setup3':
        checkAndInclude('steps/setup/step3.php');
        break;
    case 'setup4':
        checkAndInclude('steps/setup/step4.php');
        break;
    case 'setup5':
        checkAndInclude('steps/setup/step5.php');
        break;
    case 'setup6':
        checkAndInclude('steps/setup/step6.php');
        break;
    case 'setup7':
        checkAndInclude('steps/setup/step7.php');
        break;
    case 'setup8':
        checkAndInclude('steps/setup/step8.php');
        break;
    case 'migration1':
        checkAndInclude('steps/migration/step1.php');
        break;
    case 'migration2':
        checkAndInclude('steps/migration/step2.php');
        break;
    case 'migration3':
        checkAndInclude('steps/migration/step3.php');
        break;
    case 'migration4':
        checkAndInclude('steps/migration/step4.php');
        break;
    case 'migration5':
        checkAndInclude('steps/migration/step5.php');
        break;
    case 'migration6':
        checkAndInclude('steps/migration/step6.php');
        break;
    case 'migration7':
        checkAndInclude('steps/migration/step7.php');
        break;
    case 'migration8':
        checkAndInclude('steps/migration/step8.php');
        break;
    case 'upgrade1':
        checkAndInclude('steps/upgrade/step1.php');
        break;
    case 'upgrade2':
        checkAndInclude('steps/upgrade/step2.php');
        break;
    case 'upgrade3':
        checkAndInclude('steps/upgrade/step3.php');
        break;
    case 'upgrade4':
        checkAndInclude('steps/upgrade/step4.php');
        break;
    case 'upgrade5':
        checkAndInclude('steps/upgrade/step5.php');
        break;
    case 'upgrade6':
        checkAndInclude('steps/upgrade/step6.php');
        break;
    case 'upgrade7':
        checkAndInclude('steps/upgrade/step7.php');
        break;
    case 'domigration':
        checkAndInclude('steps/migration/domigration.php');
        break;
    case 'doupgrade':
        checkAndInclude('steps/upgrade/doupgrade.php');
        break;
    case 'doinstall':
        checkAndInclude('steps/setup/doinstall.php');
        break;
    case 'languagechooser':
    default:
        checkAndInclude('steps/languagechooser.php');
        break;
}

?>