<?php
/**
 * Project:
 * DZIRcms Content Management System
 *
 * Description:
 * Defines all general variables of DZIRcms.
 *
 * NOTE: This configuration file was generated by DZIRcms setup!
 *       If you want to modify the configurations for some reason, create a file
 *       "config.local.php" in "/admin/includes/" and define your own settings.
 *
 * Requirements:
 * @con_php_req 5.0
 *
 *
 * @package    DZIRcms configuration
 * @version    1.1.1
 * @author     unknown
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 *
 * {@internal
 *   created  2010-05-16
 *   modified 2010-05-20, Murat Purc, removed request check during processing ticket [#CON-307]
 *   modified 2011-03-03, Murat Purc, Added new database settings configuration
 *   modified 2011-08-24, Dominik Ziegler, changed default server path and web url definitions [#CON-430]
 *
 *   $Id$
 * }}
 *
 */


defined('CON_FRAMEWORK') or die('Illegal call');


global $cfg;

/* Section 1: Path settings
 * ------------------------
 *
 * Path settings which will vary along different DZIRcms settings.
 *
 * A little note about web and server path settings:
 * - A Web Path can be imagined as web addresses. Example:
 *   http://192.168.1.1/test/
 * - A Server Path is the path on the server's hard disk. Example:
 *   /var/www/html/admin    for Unix systems OR
 *   c:/htdocs/admin        for Windows systems
 */

/* The root server path where all frontends reside */
$cfg['path']['frontend']                = '{CONTENIDO_ROOT}';

/* The root server path to the contenido backend */
$cfg['path']['contenido']               = $cfg['path']['frontend'] . '/admin/';

/* The root server path to the conlib directory */
$cfg['path']['phplib']                  = $cfg['path']['frontend'] . '/conlib/';

/* The root server path to the pear directory */
$cfg['path']['pear']                    = $cfg['path']['frontend'] . '/pear/';

/* The server path to all WYSIWYG-Editors */
$cfg['path']['all_wysiwyg']             = $cfg['path']['contenido']  . 'external/wysiwyg/';

/* The server path to the desired WYSIWYG-Editor */
$cfg['path']['wysiwyg']                 = $cfg['path']['all_wysiwyg'] . 'tinymce3/';

/* The web server path to the DZIRcms backend */
$cfg['path']['contenido_fullhtml']      = '{CONTENIDO_WEB}/admin/';

/* The web path to all WYSIWYG-Editors */
$cfg['path']['all_wysiwyg_html']        = $cfg['path']['contenido_fullhtml'] . 'external/wysiwyg/';

/* The web path to the desired WYSIWYG-Editor */
$cfg['path']['wysiwyg_html']            = $cfg['path']['all_wysiwyg_html'] . 'tinymce3/';



/* Section 2: Database settings
 * ----------------------------
 */

/**
 * Extended database settings for MySQL/MySQLi. Note that we don't support
 * other databases in this release. These settings will be used since Contenido 4.8.15.
 *
 * NOTE: Configuration below ($contenido_host, $contenido_database, etc.)
 *       is still available because of backwards compatibility.
 *
 * @since  Contenido version 4.8.15
 */
$cfg['db'] = array(
    'connection' => array(
        'host'     => '{MYSQL_HOST}',
        'database' => '{MYSQL_DB}',
        'user'     => '{MYSQL_USER}',
        'password' => '{MYSQL_PASS}',
        'encoding' => '{DB_ENCODING}',
        'prefix'   => '{MYSQL_PREFIX}',
        'extension' => '{DB_EXTENSION}'
    ),
    'nolock'          => {NOLOCK}, // (bool) Flag to not lock tables
    'sequenceTable'   => '',       // (string) will be set later in startup!
    'haltBehavior'    => 'report', // (string) Feasible values are 'yes', 'no' or 'report'
    'haltMsgPrefix'   => (isset($_SERVER['REQUEST_URI'])) ? $_SERVER['REQUEST_URI'] . ' ' : '',
    'enableProfiling' => false,    // (bool) Flag to enable profiling
);

/*
 * Database settings.
 */

/* The prefix for all DZIRcms system tables, usually "admin" */
$cfg['sql']['sqlprefix'] = $cfg['db']['connection']['prefix'];

/* The host where your database runs on */
$contenido_host = $cfg['db']['connection']['host'];

/* The database name which you use */
$contenido_database = $cfg['db']['connection']['database'];

/* The username to access the database */
$contenido_user = $cfg['db']['connection']['user'];

/* The password to access the database */
$contenido_password = $cfg['db']['connection']['password'];

$cfg["database_encoding"] = $cfg['db']['connection']['encoding'];

$cfg["database_extension"] = $cfg['db']['connection']['extension'];

$cfg["nolock"] = $cfg['db']['nolock'];

$cfg["is_start_compatible"] = {START_COMPATIBLE};

?>